﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using System.Windows.Input;
using Buddy.Coroutines;
using Clio.Utilities;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupo.Helpers;
using Newtonsoft.Json;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupo.Rotations
{

    internal class Ninja : KupoRoutine
    {
        //Constant Variables
        private const int HutonRecast = 10000;
        private const int MinMobHp = 1000;
        private const int ArmorCrushRecast = 25000;
        private const int HutonPriority = ArmorCrushRecast + 2000;
        private const int UseGoadTP = 600;

        public SpellData SkillBlocker;

        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Ninja }; }
        }

        public static BattleCharacter _goadtarget;
        private static List<BattleCharacter> _partyList;


        public BattleCharacter Goadtarget()
        {

            _partyList = new List<BattleCharacter>();
            _partyList.AddRange(PartyManager.VisibleMembers.Select(o => o.GameObject as BattleCharacter));
            _partyList = _partyList.Where(u => Core.Player.Distance(u) <= 30 && u.InLineOfSight()).ToList();

            _goadtarget = (from battleCharacter in _partyList
                           orderby battleCharacter.CurrentTP ascending
                           where battleCharacter.CurrentTP < 1000 && battleCharacter.CurrentHealth > 1 &&
                                   (
                                   battleCharacter.CurrentJob == ClassJobType.Warrior ||
                                   battleCharacter.CurrentJob == ClassJobType.Paladin ||
                                   battleCharacter.CurrentJob == ClassJobType.Ninja ||
                                   battleCharacter.CurrentJob == ClassJobType.Dragoon ||
                                   battleCharacter.CurrentJob == ClassJobType.Monk ||
                                   battleCharacter.CurrentJob == ClassJobType.Bard ||
                                   battleCharacter.CurrentJob == ClassJobType.Rogue ||
                                   battleCharacter.CurrentJob == ClassJobType.Lancer ||
                                   battleCharacter.CurrentJob == ClassJobType.Pugilist ||
                                   battleCharacter.CurrentJob == ClassJobType.DarkKnight ||
                                   battleCharacter.CurrentJob == ClassJobType.Machinist ||
                                   battleCharacter.CurrentJob == ClassJobType.Archer
                                   )
                           select battleCharacter).FirstOrDefault();

            return _goadtarget;
        }

        public override float PullRange
        {
            get { return 2.5f; }
        }

        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return new PrioritySelector(

                Spell.Apply("Kiss of the Wasp", r => (Core.Player.ClassLevel >= 36 || !Actionmanager.HasSpell("Kiss of the Viper")), r => Core.Player),
                Spell.Apply("Kiss of the Viper", r => Actionmanager.HasSpell("Kiss of the Viper") && Core.Player.ClassLevel < 36, r => Core.Player),
                SummonChocobo()
                );
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }


        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(
                Spell.Cast("Invigorate", r => Core.Player.CurrentTP < 470, r => Core.Player)


                );
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit"),

                        Spell.PullCast("Spinning Edge")

                        )));
        }


        private readonly SpellData Jin = DataManager.GetSpellData(2263);
        private readonly SpellData Chi = DataManager.GetSpellData(2261);
        private readonly SpellData Ten = DataManager.GetSpellData(2259);
        private readonly SpellData Ninjutsu = DataManager.GetSpellData(2260);

        private readonly SpellData Kassatsu = DataManager.GetSpellData(2264);

        private readonly SpellData Trick_Attack = DataManager.GetSpellData(2258);
        private readonly SpellData Sneak_Attack = DataManager.GetSpellData(2250);

        public static HashSet<uint> OverrideBackstabIds = new HashSet<uint>()
        {
            3240//Cloud of darkness
        };

        private async Task<bool> DoNinjutsu()
        {


            //Exit early if player was inputting something
            if (Core.Player.HasAura("Mudra"))
                return true;

            if (Actionmanager.CanCastOrQueue(Jin, null))
            {
                if (!Core.Player.HasAura("Huton", true, HutonRecast))
                {
                    await CastHuton();
                    return false;
                }

                var curTarget = Core.Target as BattleCharacter;
                if (curTarget == null)
                    return false;

                if (curTarget.TimeToDeath() <= 3)
                    return false;

                //Suiton
                var taCD = Trick_Attack.Cooldown;
                //We can start casting suiton before trick attack is ready cause its going to take some time
                if (taCD.TotalMilliseconds <= 1300)
                {
                    if (!await CastSuiton())
                        return false;

                    if (!await CastTrickAttack())
                        return false;

                    if (!(Kassatsu.Cooldown.TotalMilliseconds <= 0) || !Core.Player.HasTarget)
                        return false;

                    if (await Coroutine.Wait(5000, () => Actionmanager.DoAction(Kassatsu, null)))
                    {
                        await CastRaiton();
                    }


                    return false;

                }

                if (taCD.TotalSeconds >= 20)
                {
                    await CastRaiton();
                }


                return false;
            }



            if (Actionmanager.CanCastOrQueue(Chi, null))
            {
                await CastRaiton();
                return false;
            }

            if (Actionmanager.CanCastOrQueue(Ten, null))
            {
                await Coroutine.Wait(5000, () => Actionmanager.DoAction(Ten, null));
                await CastNinjutsu();
                return false;
            }




            return false;
        }

        private async Task CastHuton()
        {
            if (await Coroutine.Wait(5000, () => Actionmanager.DoAction(Jin, null)))
            {
                if (await Coroutine.Wait(5000, () => Actionmanager.DoAction(Chi, null)))
                {
                    if (await Coroutine.Wait(5000, () => Actionmanager.DoAction(Ten, null)))
                    {
                        await CastNinjutsu();
                    }
                }
            }
        }


        private async Task<bool> CastTrickAttack()
        {

            while (Core.Player.HasAura("Suiton"))
            {
                if (Core.Player.HasTarget)
                {
                    if (OverrideBackstabIds.Contains(Core.Target.NpcId) || Core.Target.IsBehind)
                    {
                        Actionmanager.DoAction(Trick_Attack, Core.Target);
                    }
                    else if (!Core.Target.IsBehind && !Core.Target.IsFlanking)
                    {
                        Actionmanager.DoAction(Sneak_Attack, Core.Target);
                    }
                }
                if (!Core.Player.InCombat)
                    return false;

                await Coroutine.Yield();
            }

            if (!BotManager.Current.IsAutonomous)
            {
                return await Coroutine.Wait(2000, () => Core.Target != null && Core.Target.IsValid && Core.Target.HasAura("Vulnerability Up"));
            }

            return false;
        }


        private async Task<bool> CastRaiton()
        {
            if (!await Coroutine.Wait(5000, () => Actionmanager.DoAction(Ten, null))) return false;
            if (await Coroutine.Wait(5000, () => Actionmanager.DoAction(Chi, null)))
            {
                return await CastNinjutsu();
            }
            return false;
        }

        private async Task<bool> CastSuiton()
        {
            if (!await Coroutine.Wait(5000, () => Actionmanager.DoAction(Ten, null))) return false;
            if (!await Coroutine.Wait(5000, () => Actionmanager.DoAction(Chi, null))) return false;
            if (!await Coroutine.Wait(5000, () => Actionmanager.DoAction(Jin, null))) return false;


            if (await CastNinjutsu())
            {
                return await Coroutine.Wait(5000, () => Core.Player.HasAura("Suiton"));
            }

            return false;
        }


        private async Task<bool> CastNinjutsu()
        {
            if (await Coroutine.Wait(5000, () => Core.Player.HasAura("Mudra")))
            {
                bool possibly = false;
                while (Core.Player.HasAura("Mudra"))
                {
                    if (Core.Player.HasTarget)
                    {
                        if (Actionmanager.DoAction(Ninjutsu, Core.Target))
                        {
                            possibly = true;
                        }
                    }
                    if (!Core.Player.InCombat)
                        return false;

                    await Coroutine.Yield();
                }
                await Coroutine.Wait(5000, () => Ninjutsu.Cooldown.TotalSeconds > 10);
                return possibly;
            }
            return false;
        }


        private bool HasBleedingDebuff(BattleCharacter r)
        {

            return r.HasAura("Storm's Eye", false, 2000) || r.HasAura("Dancing Edge", false, 2000);

        }

        private bool ShouldMutilate(BattleCharacter r)
        {
            var name = Actionmanager.LastSpell.Name;
            if ((name == "Aeolian Edge" || name == "Shadow Fang" || name == "Dancing Edge"))
            {
                if (HasBleedingDebuff(r))
                {
                    if (r.HasAura("Shadow Fang", true, 2000))
                    {
                        return true;
                    }

                }
            }


            return false;

            //!(r as BattleCharacter).HasAura("Mutilation", true, 3500) && 

        }


        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit"),


                        Spell.Apply("Kiss of the Wasp", r => (Core.Player.ClassLevel >= 36 || !Actionmanager.HasSpell("Kiss of the Viper")), r => Core.Player),
                        Spell.Apply("Kiss of the Viper", r => Actionmanager.HasSpell("Kiss of the Viper") && Core.Player.ClassLevel < 36, r => Core.Player),

                        new ActionRunCoroutine(r => DoNinjutsu()),
                        

                        //Modified Settings
                        //--
                        Spell.Apply(r => "Armor Crush", r => !Core.Me.HasAura("Duality") && !Core.Me.HasAura("Huton", true, ArmorCrushRecast) && Actionmanager.LastSpell.Name == "Gust Slash" && Core.Me.HasAura("Huton", true, 2000) && Core.Me.ClassLevel >= 54),
                        //Buffs 
                        Spell.Cast("Blood for Blood", r => !Core.Me.HasAura("Blood for Blood") && Core.Me.CurrentTarget.CurrentHealth >= MinMobHp, r => Core.Me),
                        Spell.Cast("Internal Release", r => !Core.Me.HasAura("Internal Release") && Core.Me.CurrentTarget.CurrentHealth >= MinMobHp, r => Core.Me),
                        //--
                        Spell.Apply(r => "Duality", r => Core.Me.CurrentTarget.HasAura("Dancing Edge", true, 4000) && (Actionmanager.LastSpell.Name == "Gust Slash" && Core.Me.HasAura("Huton", true, HutonPriority) && Core.Me.ClassLevel >= 58), r => Core.Me), //Buff Aeolian Edge
                        //--
                        Spell.Cast("Assassinate", r => !Core.Me.HasAura("Duality")),
                        Spell.Cast("Dancing Edge", r =>
                        Actionmanager.LastSpell.Name == "Gust Slash" && !Core.Me.CurrentTarget.HasAura("Dancing Edge", true, 4000) && !Core.Me.CurrentTarget.HasAura("Storm's Eye", false) && !Core.Me.HasAura("Duality") && Core.Me.CurrentTarget.CurrentHealth >= MinMobHp),
                        Spell.Cast("Shadow Fang", r => Core.Me.CurrentTarget.HasAura("Dancing Edge", true, 4000) && Actionmanager.LastSpell.Name == "Spinning Edge" && !Core.Me.CurrentTarget.HasAura("Shadow Fang", true, 500) && Core.Me.HasAura("Huton", true, HutonPriority) && !Core.Me.HasAura("Duality") && Core.Me.CurrentTarget.CurrentHealth >= MinMobHp),
                        Spell.Cast("Aeolian Edge", r => Actionmanager.LastSpell.Name == "Gust Slash"),
                        //--
                        Spell.Cast("Mutilate", r => !Core.Me.CurrentTarget.HasAura("Mutilation", true, 1000) && Actionmanager.LastSpell.Name != "Gust Slash" && Actionmanager.LastSpell.Name != "Spinning Edge" && !Core.Me.HasAura("Duality") && Core.Me.CurrentTarget.CurrentHealth >= MinMobHp),
                        //--
                        Spell.Cast("Gust Slash", r => Actionmanager.LastSpell.Name == "Spinning Edge"),
                        Spell.Cast("Assassinate", r => !Core.Me.HasAura("Duality")),
                        Spell.Cast("Mug", r => Actionmanager.LastSpell.Name != "Duality"),
                        Spell.Cast("Spinning Edge"),
                        Spell.Cast("Dream Within a Dream", r => !Core.Me.HasAura("Duality")),
                //------------------------------------------------------------------------------

                //Goad				
                new Decorator(r =>
                    Goadtarget() != null &&
                    (SkillBlocker.Cooldown.TotalMilliseconds >= 1000 || Core.Me.CurrentTP <= 50) &&
                    Actionmanager.LastSpell.Name == "Ninjutsu" && !Core.Me.HasAura("Mudra"),

                    new PrioritySelector(
                        Spell.Cast("Goad", r => Goadtarget().CurrentTP <= UseGoadTP, r => Goadtarget())
                        )
                    )

                    )
                )

            );
        }

    }

}
